/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>
#include <time.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_static_states.h"

#include "initng_cron.h"

/*s_entry FIXED = { "fixed", STRING, 5, CRON_OPT }; */
s_entry MONTHLY = { "monthly", SET, 7, CRON_OPT };
s_entry WEEKLY = { "weekly", SET, 6, CRON_OPT };
s_entry DAILY = { "daily", SET, 5, CRON_OPT };
s_entry HOURLY = { "hourly", SET, 6, CRON_OPT };
s_entry LAST_RUN = { NULL, INT, 0, INTERNAL_OPT };

static void run_cron(active_h * service)
{
    if (!mark_service(service, &RUNNING))
        return;

    launch(service, T_CRON);
}

static void got_alarm(void)
{
    active_h *current = NULL;

    while_active_db(current)
    {
        time_t last;
        time_t now;

        if (current->type != TYPE_CRON)
            continue;
        now = time(0);
        last = (time_t) active_db_get_int(&LAST_RUN, current);

        if (active_db_is(&HOURLY, current))
        {
            /* 60 seconds one minute */
            /* 60 minutes, one hour */
            if (now - last > (60 * 60))
                run_cron(current);
            else
                alarm(now - last);
            continue;
        }

        if (active_db_is(&DAILY, current))
        {
            /* 60 seconds one minute */
            /* 60 minutes, one hour */
            if (now - last > (60 * 60 * 24))
                run_cron(current);
            else
                alarm(now - last);
            continue;
        }

        if (active_db_is(&WEEKLY, current))
        {
            /* 60 seconds one minute */
            /* 60 minutes, one hour */
            if (now - last > (60 * 60 * 24 * 7))
                run_cron(current);
            else
                alarm(now - last);
            continue;
        }

        if (active_db_is(&MONTHLY, current))
        {
            /* 60 seconds one minute */
            /* 60 minutes, one hour */
            if (now - last > (60 * 60 * 24 * 7 * 30))
                run_cron(current);
            else
                alarm(now - last);
            continue;
        }

    }
}

static int killed_cron(active_h * service, process_h * process)
{
    if (process->pt != T_CRON)
        return (FALSE);

    if (process->r_code == 0)
        printf("Cron for service %s, exited sucessfully!\n", service->name);
    else
        printf("Cron for service %s, exited with r_code %i!\n", service->name,
               process->r_code);


    list_del(&process->list);
    process_db_free(process);

    if (!mark_service(service, &WAIT_FOR_CRON))
    {
        F_("Unable to reset cron.\n");
        return (FALSE);
    }

    return (TRUE);
}

static int set_alarm(active_h * service)
{
    if (!IS_MARK(service, &WAIT_FOR_CRON))
        return (TRUE);

    if (service->type != TYPE_CRON)
    {
        F_("Service %s has status CRON_WAIT but is not a TYPE_CRON!\n");
        return (FALSE);
    }


    if (active_db_is(&HOURLY, service))
    {
        active_db_set_int(&LAST_RUN, service, time(0));
        alarm(60 * 60);
    }

    if (active_db_is(&DAILY, service))
    {
        active_db_set_int(&LAST_RUN, service, time(0));
        alarm(60 * 60 * 24);
    }

    if (active_db_is(&WEEKLY, service))
    {
        active_db_set_int(&LAST_RUN, service, time(0));
        alarm(60 * 60 * 24 * 7);
    }

    if (active_db_is(&MONTHLY, service))
    {
        active_db_set_int(&LAST_RUN, service, time(0));
        alarm(60 * 60 * 24 * 7 * 30);
    }

    return (TRUE);
}


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    /*initng_sdt_add(&FIXED); */
    initng_sdt_add(&MONTHLY);
    initng_sdt_add(&WEEKLY);
    initng_sdt_add(&DAILY);
    initng_sdt_add(&HOURLY);

    initng_add_hook(ASTATUS_CHANGE, 80, &set_alarm);
    initng_add_hook(HANDLE_KILLED, 80, &killed_cron);
    initng_add_hook(ALARM, 80, &got_alarm);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    /*initng_sdt_del(&FIXED); */
    initng_sdt_del(&MONTHLY);
    initng_sdt_del(&WEEKLY);
    initng_sdt_del(&DAILY);
    initng_sdt_del(&HOURLY);
    initng_del_hook(ASTATUS_CHANGE, &set_alarm);
    initng_del_hook(HANDLE_KILLED, &killed_cron);
    initng_del_hook(ALARM, &got_alarm);
}
